// this is gar.h: the generic ask routine

#include <vector>
#include <iostream>

#include "readln.h"

// *** user-imposed constraints and parameters: ***

class CConstraints {
 public:
    bool (*check)(const void *, const void *);
    const void *args;
    string text;
	
    CConstraints(bool (*check_in)(const void *, const void *),
                 const void *args_in, const string& text_in);
};

typedef vector<CConstraints> VConstraints;

typedef void CAskPars;

// *** inline implementation: ***

#define ASK_DEF_GIVEN     1
#define ASK_DEF_NOT_SHOW  2
#define ASK_WORD          4

template <class T>
T myask( const string& quest, const int flags, 
         const T& defval, const string helptext, 
         char decode( const string& resp2, T *rv2 ) )
{
    string prompt, resp;
    char test;
    T rval;

    // check default:
    bool defvalid = flags & ASK_DEF_GIVEN;

    prompt = quest;
    if (defvalid && !(flags & ASK_DEF_NOT_SHOW))
        prompt += " [" + strg(defval) + "] ? ";
    else
        prompt += " ";

    bool ok; 
    do {
        if (flags & ASK_WORD)
            resp = NRead::readwd(prompt);
        else
            resp = NRead::readln(prompt);
        test = decode(resp.c_str(), &rval );
        ok = 1;
        switch (test) {
        case 'v':
            break;
        case 'd':
            if (defvalid) {
                rval = defval;
                break;
            } 
            cout << "no default available\n";
        case 'h':
        default:
            cout << "-> please enter " << helptext;
            if (defvalid)
                cout << ";\n"
                    "-> just enter RETURN to take the default ["
                     << strg(defval) << "]";
            cout << ".\n";
            ok = 0;
        }
    } while (!ok);
    return rval;
}
