#include <string>
#include <fstream>
#include <vector>

#define SQR(a) ((a)*(a))
#define uint unsigned int
#define ulong unsigned long

using namespace std;

//! convert to string
string strg(const string& val);
string strg(const bool& val);
string strg(const int& val);
string strg(const uint& val);
string strg(const ulong& val);
string strg(const char& val);
string strg(const double& val);

//! yaml
string yaml( const string );

namespace mystd {

    //! constants
    extern const double PI;
    extern const char BEL;

    //! convert from string
    int str2vec( string inp, vector<double> *V, uint nmax=0, bool force=true );
    int any2dbl( const string inp, double *val );
    int any2int( const string inp, int *val );
    void string_extract_word( const string in, string *out1, string *out2 );
    void string_extract_line( const string in, string *out1, string *out2 );

    //! string operations
    string strip( string const& str, char const* sepSet=" \n" );

    //! improved libc calls
    void system( string cmd );

    //! time
    double justnow( void );
    string myasctime( double time, int offs );

    //! rounding
    double round_decimal( double val, double digits );

    //! vector operations
    void merge( vector<double> *V, double val );
    // bool get_const( vector<double> *V, double* val=0 );
    void unique( vector<double> *V, 
                 double tolabs=1e-100, double tolrel=1e-10 );
    int sorted( const vector<double>& V );
    bool is_equidist( double *step, const vector<double>& V );
    double sum( const vector<double>& V );

    // file names, globbing, file read
    void fname_divide( const string fname, string *fdir, string *fshort, string *fext);
    int glob_file_list(const string text, 
                       const string pre, const string ext, 
                       vector<string> *flong, vector<string> *fshort=0);
    int glob_one_file(const string text,
                      const string pre, const string ext, string *fname);
    FILE* glob_fopen(const string text, const string pre, const string ext, 
                     const char* mod, string *fname=0);
    int freadln(ifstream *f, string *s);
    int freadln(FILE *fd, string *s);

    // integration
    double Integrate( double (*func_) (double, void*), void *data_, 
                      const double low, const double hig,
                      const uint mode, const double epsabs,
                      const double epsrel );
}
