#include <stdlib.h>
#include <math.h>
#include <fftw3.h>
#include <iostream>
#include <string>
#include <vector>

using namespace std;

#include "olm.h"
#include "asi.h"
#include "special.h"

namespace NSpecial {
    void OldSimulateWithFit(void);
    void q_eval( double *par, int m_dat, double *fvec, void *data, int *info);
    void lm_print(int n_par, double *par, int m_dat, double *fvec,
                  void *data, int iflag, int iter, int nfev);
    double q_kernel( double t, double *p );
}

void NSpecial::FourierCosine(void) 
{
	NOlm::IterateD fiter;
	NOlm::SelNew();

	COld *fin, fout;
	CScan *Sin, Sout;

        fftw_plan plan;
        int n, nold=-1;

	while((fin=fiter())) {
		fout = *fin;
		fout.lDoc.push_back("delete points ...");
		fout.VS.clear();

                // *** set coordinates ***
//                olf.xco = CCoord("z", "m");
//                olf.yco = CCoord("B_dim", "B-unit");

		for (uint j=0; j<fin->VS.size(); j++) {
			Sin = &(fin->VS[j]);
			Sout = *Sin;
//			Sout.x.clear();
                        n = Sin->size();
			// Sout.y.resize(n);
                        
                        plan = fftw_plan_r2r_1d(
                                n, &(Sin->y[0]), &(Sout.y[0]),
                                FFTW_REDFT00,  FFTW_ESTIMATE);
                        nold = n; // currently not used
                        fftw_execute( plan );

                        for ( uint i=0; i<n; ++i ) {
                                Sout.y[i] /= sqrt(2.*n);
                        }

			if (Sout.size())
				fout.VS.push_back(Sout);
		}
		if (fout.VS.size())
			NOlm::OloAdd(&fout);
		else 
			cout << "no data point left\n";
	}
}

/*
void NSpecial::OldSimulateWithFit(void) 
{
    COld olf;
    static CCoord xco, yco;
    static string fnam;

    olf.name = "simul";
    olf.xco = CCoord("beta", "");
    olf.yco = CCoord("cnts", "");
    // olf.ZCo.push_back(CCoord("dim", ""));

    // CScan S;
    // S.z.push_back(0);
    // static CGrid xgrid, ygrid;

    lm_control_type control;
    lm_initialize_control(&control);
    control.epsilon = 1.e-15;
    control.ftol = 1.e-15;
    control.gtol = 1.e-15;
    control.xtol = 1.e-15;
    
    const double EPS = 1.0e-4;
    const double m_over_hbar = 2.5278e-4; // s/m * A-1
    double wgt, dkx, dky, dkz, kix, kiy, kiz, kfx, kfy, kfz, qx, qy, qz,
        p[3], fvec[3];
        
    int debug = 1;
    int nkx = 3, nky = nkx, nkz = 3001;
    double cnt_hit = 0;
    double kv = 0;
    for( int ikx=0; ikx<nkx; ++ikx ){
        dkx = 1*deg2rad/nkx;
        kix = dkx*(ikx-nkx/2);

        for( int iky=0; iky<nky; ++iky ){
            dky = 1*deg2rad/nky;
            kiy = dky*(iky-nky/2);

            for( int ikz=0; ikz<nkz; ++ikz ){
                dkz = .05*k0/nkz;
                kiz = k0+dkz*(ikz-nkz/2);

                // transform into chopper frame
                kx =  kix*ch_cos + kiz*ch_sin + kv;
                kz = -kix*ch_sin + kiz*ch_cos;
                ky =  kiy;
                // determine q-vector
                p[0] = 0.01; p[1] = 0.01; p[2] = 1;
                lm_minimize( 3, 3, p, q_eval, lm_print, NULL, &control ); 
                            // mdat, npar, ..
                if( control.info==0 || control.info>3 ){
                    // printf( "fit failed %i\n", control.info );
                    continue;
                }
                q_eval( p, 3, fvec, NULL, NULL );
                
                if( fabs(fvec[0])>EPS ||
                    fabs(fvec[1])>EPS ||
                    fabs(fvec[2])>EPS ){
                    if( debug>2 )
                        printf( "%7.4f %7.4f %7.4f bad %2i %7.4f %7.4f %7.4f"
                            " %9.4g %9.4g %9.4g\n",
                            kx, ky, kz, control.info, p[0], p[1], p[2],
                            fvec[0], fvec[1], fvec[2] );
                    continue;
                }
                printf( "%7.4f %7.4f %9.7f  %7.4f %7.4f %7.4f"
                        " %8.2g %8.2g %8.2g\n",
                        kix, ky, kz, p[0], p[1], p[2],
                        fvec[0], fvec[1], fvec[2] );
                // wgt *= exp( -SQR(kz-kPG)/2/SQR(0.03*kPG) );
                // transform into lab frame 2
                // kx -= kv;
                // transmit towards monochromator
            }
        }
    }
    // S.x = 
    // S.y = 
    // S.z[0] = 0;
    // olf.VS.push_back(S);
    // olf.VS.push_back(S);
    // NOlm::OloAdd(&olf);
}
*/
